!==============================================================================
!==============================================================================
!
!           SURFACES GENERATOR
!
!==============================================================================
!==============================================================================

program surface_generator
implicit none

integer i,i0,i1,i2,i3,ij,j,k
integer nb,type,arrow,nx,ny,nz,nnodemax,nelemtot,itr,dummy,ntrtot
integer inode,jnode,nnode_keep,surface_number
double precision x1,y1,z1
double precision x2,y2,z2
double precision x3,y3,z3
double precision xne,yne,zne,xyzn

double precision x0,y0,z0,dx,dy,dz,dist
integer,dimension(:,:),allocatable:: icon
double precision,dimension(:),allocatable::x,y,z,xn,yn,zn
double precision,dimension(:),allocatable::tempbuffer
logical debug
character :: name
!----------------------
double  precision hx,hy,hz,xind,yind,hx1,hy1
double  precision delta_x,delta_y,delta_z
integer nxmax,nymax,nzmax,nx1,ny1
!----------------------
integer nnode11,nnode12
integer nnode21
integer nnode31,nnode32

integer nelem11,nelem12
integer nelem21
integer nelem31,nelem32

integer ntr11,ntr12
integer ntr21
integer ntr31,ntr32

integer,dimension(:,:),allocatable:: iconf11,iconf12
integer,dimension(:,:),allocatable:: iconf21
integer,dimension(:,:),allocatable:: iconf31,iconf32

integer,dimension(:),allocatable:: connect11,connect12
integer,dimension(:),allocatable:: connect21
integer,dimension(:),allocatable:: connect31,connect32

double precision,dimension(:),allocatable::x11,y11,z11,x12,y12,z12
double precision,dimension(:),allocatable::x21,y21,z21
double precision,dimension(:),allocatable::x31,y31,z31,x32,y32,z32
!----------------------

inode=0
!====================== input ================================================

debug=.true.
surface_number=1
!-----------------------------------------------
! Previous input for 'L' input 

hx=1.d0
hy=1.d0
hz=0.166d0
nxmax=200
nymax=200
nzmax=40
xind=0.21d0
yind=0.33d0

!---------Input parameters calculation
delta_x=hx/(nxmax-1)
delta_y=hy/(nymax-1)
delta_z=hz/(nzmax-1)
hx1=hx-xind
nx1=int(hx1/delta_x)
hy1=hy-yind
ny1=int(hy1/delta_y)

write(*,*)'delta_x calculated: ', delta_x
write(*,*)'delta_y calculated: ', delta_y
write(*,*)'delta_z calculated: ', delta_z
write(*,*)'hx1 = ', hx1
write(*,*)'nx1 = ', nx1
write(*,*)'hy1 = ', hy1
write(*,*)'ny1 = ', ny1

!-----------------------------------------------

!face1.1 ---------------------------------------
nb=11
type=1      !1 = x fixe; 2 = y fixe; 3 = z fixe
arrow=1     !+1 = positive; -1=negative

x0=(nx1-1)*delta_x
y0=(ny1-1)*delta_y
z0=0.0d0
ny=nymax-ny1+1
nz=nzmax
dy=delta_y
dz=delta_z
nnode11=ny*nz
nelem11=(ny-1)*(nz-1)
allocate(x11(nnode11))
allocate(y11(nnode11))
allocate(z11(nnode11))
allocate(iconf11(3,2*nelem11))
allocate(connect11(nnode11))

call panel_generator (nb,type,arrow,x0,y0,z0,ny,nz,dy,dz,x11,y11,z11,iconf11)

jnode=0
do j=1,ny
   do k=1,nz
     inode=inode+1
     jnode=jnode+1
      connect11(jnode)=inode
    enddo
enddo

!!$!face1.2 ---------------------------------------
!!$nb=12
!!$type=1      !1 = x fixe; 2 = y fixe; 3 = z fixe
!!$arrow=1     !+1 = positive; -1=negative
!!$x0=0.1d0
!!$y0=0.5d0
!!$z0=0.2d0
!!$ny=4
!!$nz=3
!!$dy=0.1d0
!!$dz=0.1d0
!!$
!!$nnode12=ny*nz
!!$nelem12=(ny-1)*(nz-1)
!!$allocate(x12(nnode12))
!!$allocate(y12(nnode12))
!!$allocate(z12(nnode12))
!!$allocate(iconf12(3,2*nelem12))
!!$allocate(connect12(nnode12))
!!$
!!$call panel_generator (nb,type,arrow,x0,y0,z0,ny,nz,dy,dz,x12,y12,z12,iconf12)
!!$
!!$jnode=0
!!$do j=1,ny
!!$   do k=1,nz
!!$     inode=inode+1
!!$     jnode=jnode+1
!!$      connect12(jnode)=inode
!!$    enddo
!!$enddo

!face2.1 ---------------------------------------
nb=21
type=2      !1 = x fixe; 2 = y fixe; 3 = z fixe
arrow=1     !+1 = positive; -1=negative

x0=(nx1-1)*delta_x
y0=(ny1-1)*delta_y
z0=0.0d0
nx=nxmax-nx1+1
nz=nzmax
dx=delta_x
dz=delta_z
nnode21=nx*nz
nelem21=(nx-1)*(nz-1)
allocate(x21(nnode21))
allocate(y21(nnode21))
allocate(z21(nnode21))
allocate(iconf21(3,2*nelem21))
allocate(connect21(nnode21))

call panel_generator (nb,type,arrow,x0,y0,z0,nx,nz,dx,dz,x21,y21,z21,iconf21)

jnode=0
do i=1,nx
   do k=1,nz
     inode=inode+1
     jnode=jnode+1
      connect21(jnode)=inode
    enddo
enddo

!face3.1 ---------------------------------------
nb=31
type=3      !1 = x fixe; 2 = y fixe; 3 = z fixe
arrow=1     !+1 = positive; -1=negative

x0=0.0d0
y0=0.0d0
z0=hz
nx=nxmax
ny=ny1
dx=delta_x
dy=delta_y
nnode31=nx*ny
nelem31=(nx-1)*(ny-1)
allocate(x31(nnode31))
allocate(y31(nnode31))
allocate(z31(nnode31))
allocate(iconf31(3,2*nelem31))
allocate(connect31(nnode31))

call panel_generator (nb,type,arrow,x0,y0,z0,nx,ny,dx,dy,x31,y31,z31,iconf31)

jnode=0
do i=1,nx
   do k=1,ny
     inode=inode+1
     jnode=jnode+1
      connect31(jnode)=inode
    enddo
enddo

!face3.2 ---------------------------------------
nb=32
type=3      !1 = x fixe; 2 = y fixe; 3 = z fixe
arrow=1     !+1 = positive; -1=negative

x0=0.0d0
y0=(ny1-1)*delta_y
z0=hz
nx=nx1
ny=nymax-ny1+1
dx=delta_x
dy=delta_y

nnode32=nx*ny
nelem32=(nx-1)*(ny-1)
allocate(x32(nnode32))
allocate(y32(nnode32))
allocate(z32(nnode32))
allocate(iconf32(3,2*nelem32))
allocate(connect32(nnode32))

call panel_generator (nb,type,arrow,x0,y0,z0,nx,ny,dx,dy,x32,y32,z32,iconf32)

jnode=0
do i=1,nx
   do k=1,ny
     inode=inode+1
     jnode=jnode+1
      connect32(jnode)=inode
    enddo
enddo

!===================== assembling x,y,z & triangulation ======================

!nelemtot=nelem11+nelem12+nelem21+nelem31
nelemtot=nelem31+nelem32+nelem11+nelem21
ntrtot=nelemtot*2
!nnodemax=nnode11+nnode12+nnode21+nnode31
nnodemax=nnode31+nnode32+nnode11+nnode21

allocate(x(nnodemax))
allocate(y(nnodemax))
allocate(z(nnodemax))

dummy=0
do i=1,nnode11
   dummy=dummy+1
   x(dummy)=x11(i)
   y(dummy)=y11(i)
   z(dummy)=z11(i)
end do
!!$do i=1,nnode12
!!$   dummy=dummy+1
!!$   x(dummy)=x12(i)
!!$   y(dummy)=y12(i)
!!$   z(dummy)=z12(i)
!!$end do
do i=1,nnode21
   dummy=dummy+1
   x(dummy)=x21(i)
   y(dummy)=y21(i)
   z(dummy)=z21(i)
end do
do i=1,nnode31
   dummy=dummy+1
   x(dummy)=x31(i)
   y(dummy)=y31(i)
   z(dummy)=z31(i)
end do
do i=1,nnode32
   dummy=dummy+1
   x(dummy)=x32(i)
   y(dummy)=y32(i)
   z(dummy)=z32(i)
end do

allocate (icon(3,ntrtot))
itr=0

ntr11=nelem11*2
   write(*,*)'ntr11=',ntr11
   do i=1,ntr11
      itr=itr+1
      icon(1,itr)=connect11(iconf11(1,i))
      icon(2,itr)=connect11(iconf11(2,i))
      icon(3,itr)=connect11(iconf11(3,i))
   end do

!!$ntr12=nelem12*2
!!$   write(*,*)'ntr12=',ntr12
!!$   do i=1,ntr12
!!$      itr=itr+1
!!$      icon(1,itr)=connect12(iconf12(1,i))
!!$      icon(2,itr)=connect12(iconf12(2,i))
!!$      icon(3,itr)=connect12(iconf12(3,i))
!!$   end do
!!$
ntr21=nelem21*2
   write(*,*)'ntr21=',ntr21
   do i=1,ntr21
      itr=itr+1
      icon(1,itr)=connect21(iconf21(1,i))
      icon(2,itr)=connect21(iconf21(2,i))
      icon(3,itr)=connect21(iconf21(3,i))
   end do

ntr31=nelem31*2
   write(*,*)'ntr31=',ntr31
   do i=1,ntr31
      itr=itr+1
      icon(1,itr)=connect31(iconf31(1,i))
      icon(2,itr)=connect31(iconf31(2,i))
      icon(3,itr)=connect31(iconf31(3,i))
   end do

ntr32=nelem32*2
   write(*,*)'ntr31=',ntr32
   do i=1,ntr32
      itr=itr+1
      icon(1,itr)=connect32(iconf32(1,i))
      icon(2,itr)=connect32(iconf32(2,i))
      icon(3,itr)=connect32(iconf32(3,i))
   end do

!print*, itr, nelemtot*2

!=====[output all points (incl redundant ones) and triangles]==================

   open(unit=30,file='temp_mesh_1.vtk',status='replace')
   write(30,'(a)')'# vtk DataFile Version 3.0'
   write(30,'(a)')'surface'
   write(30,'(a)')'ASCII'
   write(30,'(a)')'DATASET UNSTRUCTURED_GRID'
   write(30,'(a7,i10,a6)')'POINTS ',nnodemax,' float'
   do i=1,nnodemax
      write(30,'(3f16.11)') x(i),y(i),z(i)
   enddo
   write(30,'(A6, 2I10)') 'CELLS ',ntrtot,4*ntrtot

   do i=1,ntrtot
     write(30,'(9I10)') 3, icon(1:3,i)-1
     ! write(*,*) icon(1:3,i)-1
   enddo
   write(30,'(A11, I10)') 'CELL_TYPES ',ntrtot
   do i=1,ntrtot
      write(30,'(I2)') 5 ! octree  (8 nodes)
   enddo
   close(30)

!==============================================================================
!=====[remove redundant points]================================================
!==============================================================================
!write(*,*)'remove redundant points'

i1=1

111 continue

do i=i1,nnodemax
   do j=i+1,nnodemax
      dist=(x(i)-x(j))**2+(y(i)-y(j))**2+(z(i)-z(j))**2
      if (dist.lt.1.d-10) then
         do ij=j+1,nnodemax
            x(ij-1)=x(ij)
            y(ij-1)=y(ij)
            z(ij-1)=z(ij)
         enddo
         do ij=1,ntrtot
            do k=1,3
               if (icon(k,ij).eq.j) icon(k,ij)=i
            enddo
         enddo
         do ij=1,ntrtot
            do k=1,3
               if (icon(k,ij).gt.j) icon(k,ij)=icon(k,ij)-1
            enddo
         enddo
         nnodemax=nnodemax-1
         i1=i
         goto 111
      endif
   enddo
enddo

!if (nnodemax/=nnode11+nnode12-3) stop 'pb with nb of points' 
nnode_keep=nnodemax

!====[resize x,y,z to correct nb of points]====================================

allocate(tempbuffer(nnode_keep))

tempbuffer=x(1:nnode_keep)
deallocate(x)
allocate(x(nnode_keep))
x=tempbuffer

tempbuffer=y(1:nnode_keep)
deallocate(y)
allocate(y(nnode_keep))
y=tempbuffer

tempbuffer=z(1:nnode_keep)
deallocate(z)
allocate(z(nnode_keep))
z=tempbuffer

deallocate(tempbuffer)

!====[output all points (without redundant ones) and triangles ]===============

   open(unit=30,file='temp_mesh_2.vtk')
   write(30,'(a)')'# vtk DataFile Version 3.0'
   write(30,'(a)')'surface'
   write(30,'(a)')'ASCII'
   write(30,'(a)')'DATASET UNSTRUCTURED_GRID'
   write(30,'(a7,i10,a6)')'POINTS ',nnode_keep,' float'
   do i=1,nnode_keep
      write(30,'(3f16.11)') x(i),y(i),z(i)
   enddo
   write(30,'(A6, 2I10)') 'CELLS ',ntrtot,4*ntrtot
   do i=1,ntrtot
      write(30,'(9I10)') 3, icon(1:3,i)-1
   enddo
   write(30,'(A11, I10)') 'CELL_TYPES ',ntrtot
   do i=1,ntrtot
      write(30,'(I2)') 5 ! octree  (8 nodes)
   enddo
   close(30)

!=====[compute normals]========================================================

allocate(xn(nnode_keep))
allocate(yn(nnode_keep))
allocate(zn(nnode_keep))

xn=0.d0
yn=0.d0
zn=0.d0
do i=1,ntrtot
   i1=icon(1,i)
   i2=icon(2,i)
   i3=icon(3,i)
   if (i1.eq.i2 .and. i2.eq.i3) print*,'element ',i,' collapsed: ',icon(:,i)
   x1=x(i1)
   x2=x(i2)
   x3=x(i3)
   y1=y(i1)
   y2=y(i2)
   y3=y(i3)
   z1=z(i1)
   z2=z(i2)
   z3=z(i3)
   xne=(y2-y1)*(z3-z1)-(y3-y1)*(z2-z1)
   yne=(z2-z1)*(x3-x1)-(z3-z1)*(x2-x1)
   zne=(x2-x1)*(y3-y1)-(x3-x1)*(y2-y1)
   xyzn=sqrt(xne**2+yne**2+zne**2)
   xne=xne/xyzn
   yne=yne/xyzn
   zne=zne/xyzn
   do j=1,3
      ij=icon(j,i)
      xn(ij)=xn(ij)+xne
      yn(ij)=yn(ij)+yne
      zn(ij)=zn(ij)+zne
   enddo
enddo
do i=1,nnode_keep
   xyzn=sqrt(xn(i)**2+yn(i)**2+zn(i)**2)
   xn(i)=xn(i)/xyzn
   yn(i)=yn(i)/xyzn
   zn(i)=zn(i)/xyzn
enddo

!=====[output points, triangles, and normals in vtk file]======================

if (debug) then
    open(unit=30,file='mesh.vtk')
    write(30,'(a)')'# vtk DataFile Version 3.0'
    write(30,'(a)')'surface'
    write(30,'(a)')'ASCII'
    write(30,'(a)')'DATASET UNSTRUCTURED_GRID'
    write(30,'(a7,i10,a6)')'POINTS ',nnode_keep,' float'
    do i=1,nnode_keep
       write(30,'(3f16.11)') x(i),y(i),z(i)
    enddo
    write(30,'(A6, 2I10)') 'CELLS ',ntrtot,4*ntrtot
    do i=1,ntrtot
        write(30,'(9I10)') 3, icon(1:3,i)-1
    enddo
    write(30,'(A11, I10)') 'CELL_TYPES ',ntrtot
    do i=1,ntrtot
       write(30,'(I2)') 5 ! octree  (8 nodes)
    enddo
    write(30,'(a11,i10)')'POINT_DATA ',nnode_keep
    write(30,'(a)')'VECTORS norm float'
    do i=1,nnode_keep
       write(30,'(3f18.13)') xn(i),yn(i),zn(i)
    enddo
    close(30)
end if

!=====[output vtk file of surface]=============================================

write(name,'(i1)') surface_number

open(unit=30,file='surface_sg_'//name//'.vtk')
write(30,'(a)')'# vtk DataFile Version 3.0'
write(30,'(a)')'surface'
write(30,'(a)')'ASCII'
write(30,'(a)')'DATASET UNSTRUCTURED_GRID'
write(30,'(a7,i10,a6)')'POINTS ',nnode_keep,' float'
do i=1,nnode_keep
   write(30,'(3f16.11)') x(i),y(i),z(i)
enddo
write(30,'(A6, 2I10)') 'CELLS ',ntrtot,4*ntrtot
do i=1,ntrtot
   write(30,'(9I10)') 3, icon(1:3,i)-1
enddo
write(30,'(A11, I10)') 'CELL_TYPES ',ntrtot
do i=1,ntrtot
   write(30,'(I2)') 5 ! octree  (8 nodes)
enddo
write(30,'(a11,i10)')'POINT_DATA ',nnode_keep
write(30,'(a)')'VECTORS norm float'
do i=1,nnode_keep
   write(30,'(3f18.13)') xn(i),yn(i),zn(i)
enddo
close(30)

!==============================================================================
!=====[produce data file readable by DOUAR]====================================
!==============================================================================

write(name,'(i1)') surface_number

open (71,file='surface-'//name//'.txt',status='unknown')
write (71,*) nnode_keep,ntrtot

do i=1,nnode_keep
   write (71,*) x(i),y(i),z(i),xn(i),yn(i),zn(i)
enddo

do i=1,ntrtot
   write (71,*) (icon(j,i),j=1,3)
enddo

close (71)

end program surface_generator

!==============================================================================
!==============================================================================
!==============================================================================
!==============================================================================
!==============================================================================
!==============================================================================
!==============================================================================
!==============================================================================
!==============================================================================
!==============================================================================
!==============================================================================
!==============================================================================
!==============================================================================
subroutine panel_generator (nb,xyz,arrow,x0,y0,z0,n1,n2,dn1,dn2,x,y,z,iconf)
implicit none

integer i,j,k,xyz,n1,n2,ip,ntr,itr,iitrii,nb
integer iconf(3,2*(n1-1)*(n2-1))
integer nnode,nelem,inode,jnode
integer arrow
!integer connect(n1*n2)

double precision x0,y0,z0,dn1,dn2
double precision x(n1*n2),y(n1*n2),z(n1*n2)
double precision,dimension(:),allocatable::xf,yf,zf

character :: name


write(*,'(a,i1)') 'face type : ',xyz
write(*,'(a,f20.15)') 'x0=',x0
write(*,'(a,f20.15)') 'y0=',y0
write(*,'(a,f20.15)') 'z0=',z0
write(*,'(a,i5)')     'n1=',n1
write(*,'(a,i5)')     'n2=',n2

nnode = n1*n2
nelem = (n1-1)*(n2-1)

!write(*,'(a,i5)')     'nnode=',nnode
!write(*,'(a,i5)')     'nelem=',nelem

!allocate(x(nnode))
!allocate(y(nnode))
!allocate(z(nnode))

!allocate(connect(n1*n2))
allocate(xf(n1*n2))
allocate(yf(n1*n2))
allocate(zf(n1*n2))

!====================== place points in space ================================

inode=0

!========== xyz=1 (x=x0) =======================================================

if (xyz.eq.1) then
jnode=0
do j=1,n1
   do k=1,n2
      inode=inode+1
      jnode=jnode+1
      x(inode)=x0
      y(inode)=y0+(j-1)*dn1
      z(inode)=z0+(k-1)*dn2
!      connect(jnode)=inode
      xf(jnode)=x0
      yf(jnode)=y0+(j-1)*dn1
      zf(jnode)=z0+(k-1)*dn2
   enddo
enddo
endif

!========== xyz=2 (y=y0) =======================================================

if (xyz.eq.2) then
jnode=0
do i=1,n1
   do k=1,n2
      inode=inode+1
      jnode=jnode+1
      x(inode)=x0+(i-1)*dn1
      y(inode)=y0
      z(inode)=z0+(k-1)*dn2
!      connect3(jnode)=inode
      xf(jnode)=x0+(i-1)*dn1
      yf(jnode)=y0
      zf(jnode)=z0+(k-1)*dn2
   enddo
enddo
endif
!========== xyz=3 (z=z0) =======================================================

if (xyz.eq.3) then
jnode=0
do i=1,n1
   do j=1,n2
      inode=inode+1
      jnode=jnode+1
      x(inode)=x0+(i-1)*dn1
      y(inode)=y0+(j-1)*dn2
      z(inode)=z0
!      connect3(jnode)=inode
      xf(jnode)=x0+(i-1)*dn1
      yf(jnode)=y0+(j-1)*dn2
      zf(jnode)=z0
   enddo
enddo
endif

if (inode/=nnode) write(*,'(a,i6)') 'pb: nb of created points',inode, ' and nnode=',nnode

!   open( unit=1234 , file='ptcles.dat', status='replace')
!   do ip=1,nnode
!      write(1234,'(3f30.15)') x(ip),y(ip),z(ip)
!   end do
!   close(1234)

!=================== Triangulate face in local numerotation ====================

ntr=2*(n1-1)*(n2-1)
!allocate(iconf(3,ntr))

iitrii=0
!========== xyz=1 (x=x0) =======================================================

if (xyz.eq.1) then
   if(arrow.lt.0) then
      itr=0
      do k=1,n2-1
         do j=1,n1-1
            itr=itr+1
            iitrii=iitrii+1
            iconf(2,itr)=(j-1)*n2+k
            iconf(1,itr)=j*n2+k
            iconf(3,itr)=j*n2+k+1
            itr=itr+1
            iitrii=iitrii+1
            iconf(2,itr)=(j-1)*n2+k
            iconf(1,itr)=j*n2+k+1
            iconf(3,itr)=(j-1)*n2+k+1
         end do
      end do
   endif
   if(arrow.gt.0) then
      itr=0
      do k=1,n2-1
         do j=1,n1-1
            itr=itr+1
            iitrii=iitrii+1
            iconf(1,itr)=(j-1)*n2+k
            iconf(2,itr)=j*n2+k
            iconf(3,itr)=j*n2+k+1
            itr=itr+1
            iitrii=iitrii+1
            iconf(1,itr)=(j-1)*n2+k
            iconf(2,itr)=j*n2+k+1
            iconf(3,itr)=(j-1)*n2+k+1
         end do
      end do
   endif
endif

!========== xyz=2 (y=y0) =======================================================

if (xyz.eq.2) then
   if(arrow.lt.0) then
      itr=0
      do k=1,n2-1
         do j=1,n1-1
            itr=itr+1
            iitrii=iitrii+1
            !   face(iitrii)=3
            iconf(1,itr)=(j-1)*n2+k
            iconf(2,itr)=j*n2+k
            iconf(3,itr)=j*n2+k+1
            itr=itr+1
            iitrii=iitrii+1
            !   face(iitrii)=3
            iconf(1,itr)=(j-1)*n2+k
            iconf(2,itr)=j*n2+k+1
            iconf(3,itr)=(j-1)*n2+k+1
         end do
      end do
   endif
   if(arrow.gt.0) then
      itr=0
      do k=1,n2-1
         do j=1,n1-1
            itr=itr+1
            iitrii=iitrii+1
            !   face(iitrii)=3
            iconf(2,itr)=(j-1)*n2+k
            iconf(1,itr)=j*n2+k
            iconf(3,itr)=j*n2+k+1
            itr=itr+1
            iitrii=iitrii+1
            !   face(iitrii)=3
            iconf(2,itr)=(j-1)*n2+k
            iconf(1,itr)=j*n2+k+1
            iconf(3,itr)=(j-1)*n2+k+1
         end do
      end do
   endif

endif

!========== xyz=3 (z=z0) =======================================================
if (xyz.eq.3) then
   if(arrow.lt.0) then
     itr=0
     do k=1,n2-1
        do j=1,n1-1
           itr=itr+1
           iitrii=iitrii+1
           !   face(iitrii)=5
           iconf(2,itr)=(j-1)*n2+k
           iconf(1,itr)=j*n2+k
           iconf(3,itr)=j*n2+k+1
           itr=itr+1
           iitrii=iitrii+1
           !   face(iitrii)=5
           iconf(2,itr)=(j-1)*n2+k
           iconf(1,itr)=j*n2+k+1
           iconf(3,itr)=(j-1)*n2+k+1
        end do
     end do
  endif

   if (arrow.gt.0) then
      itr=0
      do k=1,n2-1
         do j=1,n1-1
            itr=itr+1
            iitrii=iitrii+1
            iconf(1,itr)=(j-1)*n2+k
            iconf(2,itr)=j*n2+k
            iconf(3,itr)=j*n2+k+1
            itr=itr+1
            iitrii=iitrii+1
            iconf(1,itr)=(j-1)*n2+k
            iconf(2,itr)=j*n2+k+1
            iconf(3,itr)=(j-1)*n2+k+1
         end do
      end do
   endif
endif


call output_face(nb,n1,n2,xf,yf,zf,ntr,iconf)
return
end subroutine panel_generator

!==============================================================================
!==============================================================================
!==============================================================================
!==============================================================================
!==============================================================================

subroutine output_face (iface,n1,n2,xf,yf,zf,ntr,icon)
implicit none
integer iface,n1,n2,ntr
integer icon(3,ntr)
double precision xf(n1*n2)
double precision yf(n1*n2)
double precision zf(n1*n2)

integer i
character(len=2) :: nb

write(nb,'(i2)') iface

open(unit=30,file='face_'//nb//'.vtk')
write(30,'(a)')'# vtk DataFile Version 3.0'
write(30,'(a)')'surface'
write(30,'(a)')'ASCII'
write(30,'(a)')'DATASET UNSTRUCTURED_GRID'
write(30,'(a7,i10,a6)')'POINTS ',n1*n2,' float'

do i=1,n1*n2
   write(30,'(3f16.11)') xf(i),yf(i),zf(i)
enddo

write(30,'(A6, 2I10)') 'CELLS ',ntr,4*ntr

do i=1,ntr
   write(30,'(9I10)') 3, icon(1:3,i)-1
enddo

write(30,'(A11, I10)') 'CELL_TYPES ',ntr

do i=1,ntr
   write(30,'(I2)') 5 ! octree  (8 nodes)
 enddo

!      write(30,'(a11,i10)')'POINT_DATA ',real_nnode
!      write(30,'(a)')'VECTORS norm float'
!      do i=1,nnode
!         if (yesno_point(i)) write(30,'(3f18.13)')xn(i),yn(i),zn(i)
!      enddo
!      do i=1,real_nnode
!         write(30,'(3f18.13)') x_surfn(i),y_surfn(i),z_surfn(i)
!      enddo
close(30)

return
end subroutine output_face
