!==============================================================================
!==============================================================================
!
!     SURFACE GENERATOR
!
!==============================================================================
!==============================================================================

program surface_generator
implicit none

integer, parameter :: nrow = 9
integer lnew
integer ier,ntriangles
integer nx,ny,nz,nnode,nnodemax,nelem,nelem_surf,real_nnode,nnode_surf
integer inode,ielem,i1,ij,k,i,j,i2,i3,ie,ip,jnode,itr,iitrii
integer ntr1,ntr2,ntr3,ntr4,ntr5,ntr6,iface,surf_id

integer, dimension(:), allocatable :: effect_num
integer,dimension(:,:),allocatable::icon


double precision xc,yc,zc,sc,radius
double precision x1,y1,z1
double precision x2,y2,z2
double precision x3,y3,z3

double precision xne,yne,zne,x123,y123,z123
double precision xl,yl,zl
double precision dz,w,pi,wx,wy,dist,xyzn
double precision,dimension(:),allocatable::x,y,z,xn,yn,zn,noise,x_surf,y_surf,z_surf,x_surfn,y_surfn,z_surfn

double precision,dimension(:),allocatable::rlat,rlon,ds
integer, dimension(:), allocatable ::  list,lptr,lend,iwk
integer, dimension(:,:), allocatable ::  ltri

logical debug,random
logical, dimension(:), allocatable :: include_point 
character :: nb

pi=atan(1.d0)*4.d0

open (unit=12345,file='input.surf',status='old')
read(12345,*) i  ; debug=(i==1)
read(12345,*) xc
read(12345,*) yc
read(12345,*) zc
read(12345,*) nnode
read(12345,*) radius
read(12345,*) surf_id
close(123456)


!==============================================================================
!==============================================================================
!==============================================================================

write(*,'(a,f20.15)') 'xc=',xc
write(*,'(a,f20.15)') 'yc=',yc
write(*,'(a,f20.15)') 'zc=',zc
write(*,'(a,i5)')     'n=',nnode

!==============================================================================
!==============================================================================

allocate(x(nnode))
allocate(y(nnode))
allocate(z(nnode))
allocate(rlat(nnode))
allocate(rlon(nnode))

call random_number ( harvest = rlat(1:nnode) )
call random_number ( harvest = rlon(1:nnode) )

rlat = ( ( 1.0D+00 - rlat ) * (  -90.0D+00 ) +  rlat  *     90.0D+00 )
rlon = ( ( 1.0D+00 - rlon ) * ( -180.0D+00 ) +  rlon  *    180.0D+00 )

sc = atan ( 1.0D+00 ) / 45.0D+00

x = sc * rlon
y = sc * rlat

call trans ( nnode, y, x, x, y, z )







if(debug) then
   open(unit=123,file='points.xyz',status='replace')
   do i=1,nnode
      write(123,*) x(i),y(i),z(i)
   end do
   close(123)
end if

!==============================================================================
!==============================================================================
allocate(list(6*nnode))
allocate(lptr(6*nnode))
allocate(lend(  nnode))
allocate(iwk (2*nnode))
allocate(ds  (  nnode))
allocate(ltri(nrow,2*nnode-4))

call trmesh ( nnode, x, y, z, list, lptr, lend, lnew, iwk, iwk(nnode+1), ds, ier )

if ( ier == -2 ) then
   write ( *, '(a)' ) ' '
   write ( *, '(a)' ) 'TEST01 - Warning!'
   write ( *, '(a)' ) '  Error in TRMESH.'
   write ( *, '(a)' ) '  The first three nodes are collinear.'
   stop
end if

if ( 0 < ier ) then
   write ( *, '(a)' ) ' '
   write ( *, '(a)' ) 'TEST01 - Fatal error!'
   write ( *, '(a)' ) '  Error in TRMESH.'
    stop
end if

call trlist ( nnode, list, lptr, lend, nrow, ntriangles, ltri, ier )


write(*,'(a,i7)') 'number of triangles',ntriangles

if (debug) then
open(unit=234,file='triangles.dat',status='replace')
   do i=1,ntriangles
      write(234,*) i,ltri(1,i),ltri(2,i),ltri(3,i)
   end do
   close(234)
end if

!=====[compute normals of surface]=============================================

allocate(xn(nnode))
allocate(yn(nnode))
allocate(zn(nnode))

xn=0.d0
yn=0.d0
zn=0.d0
do i=1,ntriangles
   i1=ltri(1,i)
   i2=ltri(2,i)
   i3=ltri(3,i)
   if (i1.eq.i2 .and. i2.eq.i3) print*,'element ',i,' collapsed: ',icon(:,i)
   x1=x(i1)
   x2=x(i2)
   x3=x(i3)
   y1=y(i1)
   y2=y(i2)
   y3=y(i3)
   z1=z(i1)
   z2=z(i2)
   z3=z(i3)
   xne=(y2-y1)*(z3-z1)-(y3-y1)*(z2-z1)
   yne=(z2-z1)*(x3-x1)-(z3-z1)*(x2-x1)
   zne=(x2-x1)*(y3-y1)-(x3-x1)*(y2-y1)
   xyzn=sqrt(xne**2+yne**2+zne**2)
   xne=xne/xyzn
   yne=yne/xyzn
   zne=zne/xyzn
   do j=1,3
      ij=ltri(j,i)
      xn(ij)=xn(ij)+xne
      yn(ij)=yn(ij)+yne
      zn(ij)=zn(ij)+zne
   enddo
enddo
do i=1,nnode
   xyzn=sqrt(xn(i)**2+yn(i)**2+zn(i)**2)
   xn(i)=xn(i)/xyzn
   yn(i)=yn(i)/xyzn
   zn(i)=zn(i)/xyzn
end do

!=====[give sphere the good radius]=====
x=x*radius
y=y*radius
z=z*radius

!=====[place its center at the right place in the box]=====
x=x+xc
y=y+yc
z=z+zc

!==============================================================================
!=====[output vtk file of surface]=============================================
!==============================================================================

write(nb,'(i1)') surf_id

open(unit=30,file='surface_sg_'//nb//'.vtk')
write(30,'(a)')'# vtk DataFile Version 3.0'
write(30,'(a)')'surface'
write(30,'(a)')'ASCII'
write(30,'(a)')'DATASET UNSTRUCTURED_GRID'
write(30,'(a7,i10,a6)')'POINTS ',nnode,' float'
do i=1,nnode
   write(30,'(3f16.11)') x(i),y(i),z(i)
enddo
write(30,'(A6, 2I10)') 'CELLS ',ntriangles,4*ntriangles
do i=1,ntriangles
   write(30,'(9I10)') 3, ltri(1:3,i)-1
enddo
write(30,'(A11, I10)') 'CELL_TYPES ',ntriangles
do i=1,ntriangles
   write(30,'(I2)') 5 ! octree  (8 nodes)
enddo
write(30,'(a11,i10)')'POINT_DATA ',nnode
write(30,'(a)')'VECTORS norm float'
do i=1,nnode
   write(30,'(3f18.13)') xn(i),yn(i),zn(i)
enddo
close(30)

!==============================================================================
!=====[produce data file readable by DOUAR]====================================
!==============================================================================

write(nb,'(i1)') surf_id




end program surface_generator
